@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015  Kevin B. Hendricks, Stratford Ontario
+**  Copyright (C) 2015-2019  Kevin B. Hendricks, Stratford Ontario
 **
 **  This file is part of Sigil.
 **
@@ -120,7 +120,6 @@ GumboInterface::GumboInterface(const QString &source, const QString &version)
           m_newcsslinks(""),
           m_currentdir(""),
           m_newbody(""),
-          m_hasnbsp(false),
           m_version(version)
 {
 }
@@ -134,7 +133,6 @@ GumboInterface::GumboInterface(const QString &source, const QString &version, co
           m_newcsslinks(""),
           m_currentdir(""),
           m_newbody(""),
-          m_hasnbsp(false),
           m_version(version)
 {
 }
@@ -153,10 +151,7 @@ GumboInterface::~GumboInterface()
 void GumboInterface::parse()
 {
     if (!m_source.isEmpty() && (m_output == NULL)) {
-  
-        if (!m_version.startsWith('3')) {
-            m_hasnbsp = m_source.contains("&nbsp;");
-        }
+
         m_utf8src = m_source.toStdString();
         // remove any xml header line and any trailing whitespace
         if (m_utf8src.compare(0,5,"<?xml") == 0) {
@@ -322,6 +317,22 @@ QString GumboInterface::get_body_contents()
     return QString::fromStdString(results);
 }
 
+QString GumboInterface::get_body_text() 
+{
+    if (!m_source.isEmpty()) {
+        if (m_output == NULL) {
+            parse();
+        }
+    }
+    QList<GumboTag> tags = QList<GumboTag>() << GUMBO_TAG_BODY;
+    QList<GumboNode*> nodes = get_all_nodes_with_tags(tags);
+    if (nodes.count() != 1) {
+        return QString();
+    }
+    QString results = get_local_text_of_node(nodes.at(0));
+    return results;
+}
+
 
 QStringList GumboInterface::get_properties(GumboNode* node)
 {
@@ -536,6 +547,47 @@ QList<GumboWellFormedError> GumboInterface::error_check()
     return errlist;
 }
 
+QList<GumboWellFormedError> GumboInterface::fragment_error_check()
+{
+    QList<GumboWellFormedError> errlist;
+
+    // In case we ever have to revert to earlier versions, please note the following
+    // additional initialization is needed because Microsoft Visual Studio 2013 (and earlier?)
+    // do not properly initialize myoptions from the static const kGumboDefaultOptions defined
+    // in the gumbo library.  Instead whatever was in memory at the time is used causing random 
+    // issues later on so if reverting remember to keep these specific changes as the bug 
+    // they work around took a long long time to track down
+    GumboOptions myoptions = kGumboDefaultOptions;
+    myoptions.tab_stop = 4;
+    myoptions.use_xhtml_rules = true;
+    myoptions.stop_on_first_error = false;
+    myoptions.max_tree_depth = 400;
+    myoptions.max_errors = -1;
+
+    if (!m_source.isEmpty() && (m_output == NULL)) {
+
+        m_utf8src = m_source.toStdString();
+        m_output = gumbo_parse_fragment(&myoptions, m_utf8src.data(), m_utf8src.length(),
+					GUMBO_TAG_BODY, GUMBO_NAMESPACE_HTML);
+    }
+    const GumboVector* errors  = &m_output->errors;
+    for (unsigned int i=0; i< errors->length; ++i) {
+        GumboError* er = static_cast<GumboError*>(errors->data[i]);
+        GumboWellFormedError gperror;
+        gperror.line = er->position.line;
+        gperror.column = er->position.column;
+        // unsigned int typenum = er->type;
+        GumboStringBuffer text;
+        gumbo_string_buffer_init(&text);
+        gumbo_error_to_string(er, &text);
+        std::string errmsg(text.data, text.length);
+        gperror.message = QString::fromStdString(errmsg);
+        gumbo_string_buffer_destroy(&text);
+        errlist.append(gperror);
+    }
+    return errlist;
+}
+
 
 QList<GumboNode*> GumboInterface::get_all_nodes_with_attribute(const QString& attname)
 {
@@ -827,13 +879,6 @@ std::string GumboInterface::substitute_xml_entities_into_text(const std::string
     replace_all(result, "&", "&amp;");
     replace_all(result, "<", "&lt;");
     replace_all(result, ">", "&gt;");
-    // convert non-breaking spaces to entities to prevent their loss for later editing
-    // See the strange//buggy behaviour of Qt QTextDocument toPlainText() routine 
-    if (m_hasnbsp) {
-        replace_all(result, "\xc2\xa0", "&nbsp;");
-    } else {
-        replace_all(result, "\xc2\xa0", "&#160;");
-    }
     return result;
 }
 
