@@ -1,5 +1,6 @@
 /************************************************************************
 **
+**  Copyright (C) 2019 Kevin B. Hendricks, Stratford, Ontario, Canada
 **  Copyright (C) 2009, 2010, 2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
@@ -64,7 +65,10 @@ QString PerformCSSUpdates::operator()()
         ")");
 
     int start_index = 0;
+    // this next line is what is crashing
     QRegularExpressionMatch mo = reference.match(result, start_index);
+    // handle case if no initial match at all
+    if (!mo.hasMatch()) return result;
     do {
         bool changes_made = false;
         for (int i = 1; i <= reference.captureCount(); ++i) {
@@ -75,28 +79,31 @@ QString PerformCSSUpdates::operator()()
             int frag_start_index = 0;
             QString fragment = mo.captured(i);
             QRegularExpressionMatch frag_mo = urls.match(fragment, frag_start_index);
-            do {
-                for (int j = 1; j <= urls.captureCount(); ++j) {
-                    if (frag_mo.captured(j).trimmed().isEmpty()) {
-                        continue;
-                    }
-                    QString apath = Utility::URLDecodePath(frag_mo.captured(j));
-                    QString search_key = QDir::cleanPath(origDir + FORWARD_SLASH + apath);
-                    QString new_href;
-                    if (m_CSSUpdates.contains(search_key)) {
-                        new_href = m_CSSUpdates.value(search_key);
-                    }
-                    if (!new_href.isEmpty()) {
-                        new_href = Utility::URLEncodePath(new_href);
-                        // Replace the old url with the new one
-                        fragment.replace(frag_mo.capturedStart(j), frag_mo.capturedLength(j), new_href);
-                        changes_made = true;
-                    }
+	    // only loop if at least one match was found
+	    if (frag_mo.hasMatch()) {
+                do {
+                    for (int j = 1; j <= urls.captureCount(); ++j) {
+                        if (frag_mo.captured(j).trimmed().isEmpty()) {
+                            continue;
+                        }
+                        QString apath = Utility::URLDecodePath(frag_mo.captured(j));
+                        QString search_key = QDir::cleanPath(origDir + FORWARD_SLASH + apath);
+                        QString new_href;
+                        if (m_CSSUpdates.contains(search_key)) {
+                            new_href = m_CSSUpdates.value(search_key);
+                        }
+                        if (!new_href.isEmpty()) {
+                            new_href = Utility::URLEncodePath(new_href);
+                            // Replace the old url with the new one
+                            fragment.replace(frag_mo.capturedStart(j), frag_mo.capturedLength(j), new_href);
+                            changes_made = true;
+                        }
     
-                }
-                frag_start_index += frag_mo.capturedLength();
-                frag_mo = urls.match(fragment, frag_start_index);
-            } while (frag_mo.hasMatch());
+                    }
+                    frag_start_index += frag_mo.capturedLength();
+                    frag_mo = urls.match(fragment, frag_start_index);
+                } while (frag_mo.hasMatch());
+	    }
             // Replace the original attribute string fragment with the new one
             if (changes_made) {
                 result.replace(mo.capturedStart(i), mo.capturedLength(i), fragment);
