@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2016, 2017, 2018 Kevin B Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2016 - 2019 Kevin B Hendricks, Stratford, Ontario, Canada
 **  Copyright (C) 2012 John Schember <john@nachtimwald.com>
 **  Copyright (C) 2012  Dave Heiland
 **  Copyright (C) 2012  Grant Drake
@@ -135,6 +135,9 @@ FlowTab::~FlowTab()
         delete(m_views);
         m_views = 0;
     }
+
+    m_HTMLResource = NULL;
+
 }
 
 void FlowTab::CreateBookViewIfRequired(bool is_delayed_load)
@@ -223,6 +226,10 @@ void FlowTab::DelayedInitialization()
     // Only now will we wire up monitoring of ResourceChanged, to prevent
     // unnecessary saving and marking of the resource for reloading.
     DelayedConnectSignalsToSlots();
+
+    // use FilteredCursorMove Signal to sync Preview instead
+    // emit ScrollPreviewImmediately();
+
     // Cursor set in constructor
     QApplication::restoreOverrideCursor();
 }
@@ -507,6 +514,11 @@ void FlowTab::EmitUpdatePreviewImmediately()
     emit UpdatePreviewImmediately();
 }
 
+void FlowTab::EmitScrollPreviewImmediately()
+{
+    emit ScrollPreviewImmediately();
+}
+
 void FlowTab::EmitUpdateCursorPosition()
 {
     emit UpdateCursorPosition(GetCursorLine(), GetCursorColumn());
@@ -1265,6 +1277,8 @@ void FlowTab::InsertBulletedList()
 {
     if (m_ViewState == MainWindow::ViewState_BookView) {
         m_wBookView->ExecCommand("insertUnorderedList");
+    } else if (m_ViewState == MainWindow::ViewState_CodeView) {
+        m_wCodeView->ApplyListToSelection("ul");
     }
 }
 
@@ -1272,13 +1286,18 @@ void FlowTab::InsertNumberedList()
 {
     if (m_ViewState == MainWindow::ViewState_BookView) {
         m_wBookView->ExecCommand("insertOrderedList");
+    } else if (m_ViewState == MainWindow::ViewState_CodeView) {
+        m_wCodeView->ApplyListToSelection("ol");
     }
+
 }
 
 void FlowTab::DecreaseIndent()
 {
     if (m_ViewState == MainWindow::ViewState_BookView) {
         m_wBookView->page()->triggerAction(QWebPage::Outdent);
+    } else if (m_ViewState == MainWindow::ViewState_CodeView) {
+        m_wCodeView->WrapSelectionInElement("blockquote", true);
     }
 }
 
@@ -1286,6 +1305,8 @@ void FlowTab::IncreaseIndent()
 {
     if (m_ViewState == MainWindow::ViewState_BookView) {
         m_wBookView->page()->triggerAction(QWebPage::Indent);
+    } else if (m_ViewState == MainWindow::ViewState_CodeView) {
+        m_wCodeView->WrapSelectionInElement("blockquote", false);
     }
 }
 
@@ -1533,7 +1554,7 @@ QString FlowTab::GetCaretElementName()
     if (m_ViewState == MainWindow::ViewState_BookView) {
         return m_wBookView->GetCaretElementName();
     } else {
-        return ContentTab::GetCaretElementName();
+        return m_wCodeView->GetCaretElementName();;
     }
 }
 
@@ -1584,7 +1605,7 @@ void FlowTab::ConnectBookViewSignalsToSlots()
     connect(m_wBookView, SIGNAL(textChanged()), this, SLOT(EmitContentChanged()));
     connect(m_wBookView, SIGNAL(BVInspectElement()), this, SIGNAL(InspectElement()));
     connect(m_wBookView, SIGNAL(PageUpdated()), this, SLOT(EmitUpdatePreview()));
-    connect(m_wBookView, SIGNAL(PageClicked()), this, SLOT(EmitUpdatePreviewImmediately()));
+    connect(m_wBookView, SIGNAL(PageClicked()), this, SLOT(EmitScrollPreviewImmediately()));
     connect(m_wBookView, SIGNAL(PageOpened()), this, SLOT(EmitUpdatePreviewImmediately()));
     connect(m_wBookView, SIGNAL(DocumentLoaded()), this, SLOT(EmitUpdatePreviewImmediately()));
 }
@@ -1604,9 +1625,10 @@ void FlowTab::ConnectCodeViewSignalsToSlots()
     connect(m_wCodeView, SIGNAL(SpellingHighlightRefreshRequest()), this, SIGNAL(SpellingHighlightRefreshRequest()));
     connect(m_wCodeView, SIGNAL(ShowStatusMessageRequest(const QString &)), this, SIGNAL(ShowStatusMessageRequest(const QString &)));
     connect(m_wCodeView, SIGNAL(FilteredTextChanged()), this, SLOT(EmitContentChanged()));
-    connect(m_wCodeView, SIGNAL(FilteredCursorMoved()), this, SLOT(EmitUpdatePreview()));
+    //  This is needed to capture scroll from arrow keys and the like                        
+    connect(m_wCodeView, SIGNAL(FilteredCursorMoved()), this, SLOT(EmitScrollPreviewImmediately()));
     connect(m_wCodeView, SIGNAL(PageUpdated()), this, SLOT(EmitUpdatePreview()));
-    connect(m_wCodeView, SIGNAL(PageClicked()), this, SLOT(EmitUpdatePreviewImmediately()));
+    connect(m_wCodeView, SIGNAL(PageClicked()), this, SLOT(EmitScrollPreviewImmediately()));
     connect(m_wCodeView, SIGNAL(DocumentSet()), this, SLOT(EmitUpdatePreviewImmediately()));
     connect(m_wCodeView, SIGNAL(MarkSelectionRequest()), this, SIGNAL(MarkSelectionRequest()));
     connect(m_wCodeView, SIGNAL(ClearMarkedTextRequest()), this, SIGNAL(ClearMarkedTextRequest()));
