@@ -153,17 +153,8 @@ void BookViewPreview::CustomSetDocument(const QString &path, const QString &html
     if (!url().isEmpty()) {
         StoreCurrentCaretLocation();
         // to help keep memory footprint small clear any memory caches when a new page loads
- 	if (url().path() != path) {
+ 	if (url().toLocalFile() != path) {
 	    settings()->clearMemoryCaches();
-#if 0
-	    // An attempt to flush caches but not the font caches
-	    // These need to be kept in sync with MainWindow::clearMemoryCaches()
-	    // settings()->setObjectCacheCapacities(0,0,0);
-	    // settings()->setObjectCacheCapacities(0, 8 * 1024 * 1024, 16 * 1024 * 1024);
-            // int n = settings()->maximumPagesInCache();
-	    // settings()->setMaximumPagesInCache(0);
-	    // settings()->setMaximumPagesInCache(n);
-#endif
 	} 
     }
 
@@ -174,7 +165,7 @@ void BookViewPreview::CustomSetDocument(const QString &path, const QString &html
     // Sigil as well as catering for section splits etc.
     QString replaced_html = html;
     replaced_html = replaced_html.replace("<html>", "<html xmlns=\"http://www.w3.org/1999/xhtml\">");
-    setContent(replaced_html.toUtf8(), "application/xhtml+xml", QUrl::fromLocalFile(path));
+    setContent(replaced_html.toUtf8(), "application/xhtml+xml;charset=utf-8", QUrl::fromLocalFile(path));
 }
 
 bool BookViewPreview::IsLoadingFinished()
@@ -228,10 +219,12 @@ void BookViewPreview::mousePressEvent(QMouseEvent *event)
 void BookViewPreview::ScrollToTop()
 {
     QString caret_location = "var elementList = document.getElementsByTagName(\"body\");"
-                             "var element = elementList[0];";
+      "var element = elementList[0];";
     QString scroll = "var from_top = window.innerHeight / 2;"
+                     "if (typeof element !== 'undefined') {"
                      "$.scrollTo(element, 0, {offset: {top:-from_top, left:0} });";
-    EvaluateJavascript(caret_location % scroll % SET_CURSOR_JS);
+    QString script = caret_location + scroll + SET_CURSOR_JS + "}";
+    EvaluateJavascript(script);
 }
 
 void BookViewPreview::ScrollToFragment(const QString &fragment)
@@ -249,11 +242,12 @@ void BookViewPreview::ScrollToFragmentInternal(const QString &fragment)
         ScrollToTop();
         return;
     }
-
     QString caret_location = "var element = document.getElementById(\"" % fragment % "\");";
     QString scroll = "var from_top = window.innerHeight / 2.5;"
+                     "if (typeof element !== 'undefined') {"
                      "$.scrollTo(element, 0, {offset: {top:-from_top, left:0 } });";
-    EvaluateJavascript(caret_location % scroll % SET_CURSOR_JS);
+    QString script = caret_location + scroll + SET_CURSOR_JS + "}";
+    EvaluateJavascript(script);
 }
 
 bool BookViewPreview::FindNext(const QString &search_regex,
@@ -364,6 +358,11 @@ void BookViewPreview::SetUpFindForSelectedText(const QString &search_regex)
     // Nothing to do for Book Preview
 }
 
+void BookViewPreview::LoadingStarted()
+{
+    m_isLoadFinished = false;
+}
+
 void BookViewPreview::UpdateFinishedState(bool okay)
 {
     if (okay) {
@@ -596,8 +595,9 @@ void BookViewPreview::StoreCaretLocationUpdate(const QList<ViewEditor::ElementIn
     QString caret_location = "var element = " + GetElementSelectingJS_NoTextNodes(hierarchy) + ";";
     // We scroll to the element and center the screen on it
     QString scroll = "var from_top = window.innerHeight / 2;"
+                     "if (typeof element !== 'undefined') {"
                      "$.scrollTo( element, 0, {offset: {top:-from_top, left:0 } } );";
-    m_CaretLocationUpdate = caret_location + scroll + SET_CURSOR_JS;
+    m_CaretLocationUpdate = caret_location + scroll + SET_CURSOR_JS + "}";
 }
 
 QString BookViewPreview::GetElementSelectingJS_WithTextNode(const QList<ViewEditor::ElementIndex> &hierarchy) const
@@ -792,6 +792,7 @@ void BookViewPreview::ConnectSignalsToSlots()
 {
     connect(this,  SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(OpenContextMenu(const QPoint &)));
     connect(m_InspectElement,    SIGNAL(triggered()),  this, SLOT(InspectElement()));
+    connect(page(), SIGNAL(loadStarted()), this, SLOT(LoadingStarted()));
     connect(page(), SIGNAL(loadFinished(bool)), this, SLOT(UpdateFinishedState(bool)));
     connect(page(), SIGNAL(linkClicked(const QUrl &)), this, SIGNAL(LinkClicked(const QUrl &)));
     connect(page(), SIGNAL(loadFinished(bool)), this, SLOT(WebPageJavascriptOnLoad()));
